import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { URI } from 'vscode/vscode/vs/base/common/uri';
import { localize2, localize } from 'vscode/vscode/vs/nls';
import { Action2, MenuId, MenuRegistry, registerAction2 } from 'vscode/vscode/vs/platform/actions/common/actions';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IDialogService } from 'vscode/vscode/vs/platform/dialogs/common/dialogs.service';
import { IViewsService } from 'vscode/vscode/vs/workbench/services/views/common/viewsService.service';
import { ChatAgentLocation } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents';
import { IChatAgentService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents.service';
import { CONTEXT_CHAT_INPUT_HAS_TEXT, CONTEXT_CHAT_REQUEST_IN_PROGRESS, CONTEXT_CHAT_LOCATION, CONTEXT_IN_CHAT_INPUT, CONTEXT_LANGUAGE_MODELS_ARE_USER_SELECTABLE, CONTEXT_CHAT_INPUT_HAS_AGENT, CONTEXT_CHAT_ENABLED, CONTEXT_CHAT_EDITING_PARTICIPANT_REGISTERED } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatContextKeys';
import { applyingChatEditsContextKey } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatEditingService';
import { IChatEditingService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatEditingService.service';
import { extractAgentAndCommand, chatAgentLeader } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatParserTypes';
import { IChatService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatService.service';
import { EDITS_VIEW_ID } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/browser/chat';
import { IChatWidgetService } from 'vscode/vscode/vs/workbench/contrib/chat/browser/chat.service';
import '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/browser/actions/chatActions';
import { CHAT_CATEGORY } from 'vscode/vscode/vs/workbench/contrib/chat/browser/actions/chatConstants';

class SubmitAction extends Action2 {
    static { this.ID = 'workbench.action.chat.submit'; }
    constructor() {
        super({
            id: SubmitAction.ID,
            title: ( localize2(7542, "Send and Dispatch")),
            f1: false,
            category: CHAT_CATEGORY,
            icon: Codicon.send,
            precondition: ( (ContextKeyExpr.and(
                CONTEXT_CHAT_INPUT_HAS_TEXT,
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                 (ContextKeyExpr.or(
                     (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession)),
                     (ContextKeyExpr.and(
                         (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                         (applyingChatEditsContextKey.toNegated())
                    ))
                ))
            ))),
            keybinding: {
                when: CONTEXT_IN_CHAT_INPUT,
                primary: 3 ,
                weight: 100
            },
            menu: [
                {
                    id: MenuId.ChatExecuteSecondary,
                    group: 'group_1',
                    order: 1
                },
                {
                    id: MenuId.ChatExecute,
                    order: 4,
                    when: ( (ContextKeyExpr.and(
                         (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                         (ContextKeyExpr.or(
                             (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession)),
                             (ContextKeyExpr.and(
                                 (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                                 (applyingChatEditsContextKey.toNegated())
                            ))
                        ))
                    ))),
                    group: 'navigation',
                },
            ]
        });
    }
    run(accessor, ...args) {
        const context = args[0];
        const widgetService = accessor.get(IChatWidgetService);
        const widget = context?.widget ?? widgetService.lastFocusedWidget;
        widget?.acceptInput(context?.inputValue);
    }
}
class SubmitWithoutDispatchingAction extends Action2 {
    static { this.ID = 'workbench.action.chat.submitWithoutDispatching'; }
    constructor() {
        super({
            id: SubmitWithoutDispatchingAction.ID,
            title: ( localize2(7543, "Send")),
            f1: false,
            category: CHAT_CATEGORY,
            precondition: ( (ContextKeyExpr.and(
                CONTEXT_CHAT_INPUT_HAS_TEXT,
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                 (ContextKeyExpr.and( (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.Panel))))
            ))),
            keybinding: {
                when: CONTEXT_IN_CHAT_INPUT,
                primary: 512  | 1024  | 3 ,
                weight: 100
            },
            menu: [
                {
                    id: MenuId.ChatExecuteSecondary,
                    group: 'group_1',
                    order: 2
                }
            ]
        });
    }
    run(accessor, ...args) {
        const context = args[0];
        const widgetService = accessor.get(IChatWidgetService);
        const widget = context?.widget ?? widgetService.lastFocusedWidget;
        widget?.acceptInput(context?.inputValue, { noCommandDetection: true });
    }
}
const ChatModelPickerActionId = 'workbench.action.chat.pickModel';
MenuRegistry.appendMenuItem(MenuId.ChatExecute, {
    command: {
        id: ChatModelPickerActionId,
        title: ( localize2(7544, "Pick Model")),
    },
    order: 3,
    group: 'navigation',
    when: ( (ContextKeyExpr.and(
        CONTEXT_LANGUAGE_MODELS_ARE_USER_SELECTABLE,
         (ContextKeyExpr.or(
             (ContextKeyExpr.equals(CONTEXT_CHAT_LOCATION.key, ChatAgentLocation.Panel)),
             (ContextKeyExpr.equals(CONTEXT_CHAT_LOCATION.key, ChatAgentLocation.EditingSession)),
             (ContextKeyExpr.equals(CONTEXT_CHAT_LOCATION.key, ChatAgentLocation.Editor))
        ))
    ))),
});
class ChatSubmitSecondaryAgentAction extends Action2 {
    static { this.ID = 'workbench.action.chat.submitSecondaryAgent'; }
    constructor() {
        super({
            id: ChatSubmitSecondaryAgentAction.ID,
            title: ( localize2(7545, "Submit to Secondary Agent")),
            precondition: ( (ContextKeyExpr.and(
                CONTEXT_CHAT_INPUT_HAS_TEXT,
                 (CONTEXT_CHAT_INPUT_HAS_AGENT.negate()),
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate())
            ))),
            menu: {
                id: MenuId.ChatExecuteSecondary,
                group: 'group_1',
                order: 3
            },
            keybinding: {
                when: CONTEXT_IN_CHAT_INPUT,
                primary: 2048  | 3 ,
                weight: 100
            },
        });
    }
    run(accessor, ...args) {
        const context = args[0];
        const agentService = accessor.get(IChatAgentService);
        const secondaryAgent = agentService.getSecondaryAgent();
        if (!secondaryAgent) {
            return;
        }
        const widgetService = accessor.get(IChatWidgetService);
        const widget = context?.widget ?? widgetService.lastFocusedWidget;
        if (!widget) {
            return;
        }
        if (extractAgentAndCommand(widget.parsedInput).agentPart) {
            widget.acceptInput();
        }
        else {
            widget.lastSelectedAgent = secondaryAgent;
            widget.acceptInputWithPrefix(`${chatAgentLeader}${secondaryAgent.name}`);
        }
    }
}
class SendToChatEditingAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.chat.sendToChatEditing',
            title: ( localize2(7546, "Send to Copilot Edits")),
            precondition: ( (ContextKeyExpr.and(
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                 (CONTEXT_CHAT_INPUT_HAS_AGENT.negate()),
                CONTEXT_CHAT_INPUT_HAS_TEXT
            ))),
            category: CHAT_CATEGORY,
            f1: false,
            menu: {
                id: MenuId.ChatExecuteSecondary,
                group: 'group_1',
                order: 4,
                when: ( (ContextKeyExpr.and(
                    CONTEXT_CHAT_ENABLED,
                    CONTEXT_CHAT_EDITING_PARTICIPANT_REGISTERED,
                     (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession))
                )))
            },
            keybinding: {
                weight: 200 ,
                primary: 2048  | 512  | 3 ,
                when: ( (ContextKeyExpr.and(
                    CONTEXT_CHAT_ENABLED,
                    CONTEXT_CHAT_EDITING_PARTICIPANT_REGISTERED,
                     (CONTEXT_CHAT_LOCATION.notEqualsTo(ChatAgentLocation.EditingSession))
                )))
            }
        });
    }
    async run(accessor, ...args) {
        if (!accessor.get(IChatAgentService).getDefaultAgent(ChatAgentLocation.EditingSession)) {
            return;
        }
        const context = args[0];
        const widgetService = accessor.get(IChatWidgetService);
        const widget = context?.widget ?? widgetService.lastFocusedWidget;
        if (!widget || widget.viewModel?.model.initialLocation === ChatAgentLocation.EditingSession) {
            return;
        }
        const viewsService = accessor.get(IViewsService);
        const dialogService = accessor.get(IDialogService);
        const chatEditingService = accessor.get(IChatEditingService);
        const currentEditingSession = chatEditingService.currentEditingSessionObs.get();
        const currentEditCount = currentEditingSession?.entries.get().length;
        if (currentEditCount) {
            const result = await dialogService.confirm({
                title: ( localize(7547, "Start new editing session?")),
                message: currentEditCount === 1
                    ? ( localize(
                    7548,
                    "Starting a new editing session will end your current editing session containing {0} file. Do you wish to proceed?",
                    currentEditCount
                ))
                    : ( localize(
                    7549,
                    "Starting a new editing session will end your current editing session containing {0} files. Do you wish to proceed?",
                    currentEditCount
                )),
                type: 'info',
                primaryButton: ( localize(7550, "Yes"))
            });
            if (!result.confirmed) {
                return;
            }
            await currentEditingSession?.stop();
        }
        const { widget: editingWidget } = await viewsService.openView(EDITS_VIEW_ID);
        for (const attachment of widget.attachmentModel.attachments) {
            if (attachment.isFile && URI.isUri(attachment.value)) {
                chatEditingService.currentEditingSessionObs.get()?.addFileToWorkingSet(attachment.value);
            }
            else {
                editingWidget.attachmentModel.addContext(attachment);
            }
        }
        editingWidget.setInput(widget.getInput());
        widget.setInput('');
        widget.attachmentModel.clear();
        editingWidget.acceptInput();
        editingWidget.focusInput();
    }
}
class SendToNewChatAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.chat.sendToNewChat',
            title: ( localize2(7551, "Send to New Chat")),
            precondition: ( (ContextKeyExpr.and(
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                CONTEXT_CHAT_INPUT_HAS_TEXT
            ))),
            category: CHAT_CATEGORY,
            f1: false,
            menu: {
                id: MenuId.ChatExecuteSecondary,
                group: 'group_2'
            },
            keybinding: {
                weight: 200 ,
                primary: 2048  | 1024  | 3 ,
                when: CONTEXT_IN_CHAT_INPUT,
            }
        });
    }
    async run(accessor, ...args) {
        const context = args[0];
        const widgetService = accessor.get(IChatWidgetService);
        const widget = context?.widget ?? widgetService.lastFocusedWidget;
        if (!widget) {
            return;
        }
        widget.clear();
        widget.acceptInput(context?.inputValue);
    }
}
class CancelAction extends Action2 {
    static { this.ID = 'workbench.action.chat.cancel'; }
    constructor() {
        super({
            id: CancelAction.ID,
            title: ( localize2(7552, "Cancel")),
            f1: false,
            category: CHAT_CATEGORY,
            icon: Codicon.stopCircle,
            menu: {
                id: MenuId.ChatExecute,
                when: ( (ContextKeyExpr.or(CONTEXT_CHAT_REQUEST_IN_PROGRESS,  (ContextKeyExpr.and(
                     (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                    applyingChatEditsContextKey
                ))))),
                order: 4,
                group: 'navigation',
            },
            keybinding: {
                weight: 200 ,
                primary: 2048  | 9 ,
                win: { primary: 512  | 1  },
            }
        });
    }
    run(accessor, ...args) {
        const context = args[0];
        const widgetService = accessor.get(IChatWidgetService);
        const widget = context?.widget ?? widgetService.lastFocusedWidget;
        if (!widget) {
            return;
        }
        const chatService = accessor.get(IChatService);
        if (widget.viewModel) {
            chatService.cancelCurrentRequestForSession(widget.viewModel.sessionId);
        }
        const chatEditingService = accessor.get(IChatEditingService);
        const currentEditingSession = chatEditingService.currentEditingSession;
        if (currentEditingSession && currentEditingSession?.chatSessionId === widget.viewModel?.sessionId) {
            chatEditingService.currentAutoApplyOperation?.cancel();
        }
    }
}
function registerChatExecuteActions() {
    registerAction2(SubmitAction);
    registerAction2(SubmitWithoutDispatchingAction);
    registerAction2(CancelAction);
    registerAction2(SendToNewChatAction);
    registerAction2(ChatSubmitSecondaryAgentAction);
    registerAction2(SendToChatEditingAction);
}

export { CancelAction, ChatModelPickerActionId, ChatSubmitSecondaryAgentAction, SubmitAction, registerChatExecuteActions };
