import { Emitter } from 'vscode/vscode/vs/base/common/event';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { basename } from 'vscode/vscode/vs/base/common/resources';

class ChatAttachmentModel extends Disposable {
    constructor() {
        super(...arguments);
        this._attachments = ( new Map());
        this._onDidChangeContext = this._register(( new Emitter()));
        this.onDidChangeContext = this._onDidChangeContext.event;
    }
    get attachments() {
        return Array.from(( this._attachments.values()));
    }
    get size() {
        return this._attachments.size;
    }
    getAttachmentIDs() {
        return ( new Set(( this._attachments.keys())));
    }
    clear() {
        this._attachments.clear();
        this._onDidChangeContext.fire();
    }
    delete(variableEntryId) {
        this._attachments.delete(variableEntryId);
        this._onDidChangeContext.fire();
    }
    addFile(uri, range) {
        this.addContext(this.asVariableEntry(uri, range));
    }
    asVariableEntry(uri, range) {
        return {
            value: uri,
            id: ( uri.toString()) + (range?.toString() ?? ''),
            name: basename(uri),
            isFile: true,
            isDynamic: true
        };
    }
    addContext(...attachments) {
        for (const attachment of attachments) {
            if (!( this._attachments.has(attachment.id))) {
                this._attachments.set(attachment.id, attachment);
            }
        }
        this._onDidChangeContext.fire();
    }
    clearAndSetContext(...attachments) {
        this.clear();
        this.addContext(...attachments);
    }
}

export { ChatAttachmentModel };
